
import csv

def read_csv_file(path : str , read_encoding : str) -> dict:
    temp_dict : dict = {}
    with open(path , 'r', encoding = read_encoding , errors = 'replace') as f:
        reader = csv.reader(f)
        headers = next(reader)
        for row_num , row in enumerate(reader):
            row_dict = {}
            for i , value in enumerate(row):
                if i < len(headers):
                    row_dict[headers[i]] = value
                else:
                    row_dict[f"extra_col_{i}"] = value
            temp_dict[row_num] = row_dict
        return temp_dict


# *-* coding: utf-8 *-*
# src\__init__.py
# SHICTHRS CSV LOADER
# AUTHOR : SHICTHRS-JNTMTMTM
# Copyright : © 2025-2026 SHICTHRS, Std. All rights reserved.
# lICENSE : GPL-3.0

import os
from colorama import init
init()
from .utils.SHRCSVLoader_read_csv_file import read_csv_file
from .utils.SHRCSVLoader_write_csv_file import write_csv_file
from .utils.SHRCSVLoader_insert_csv_file import insert_header_to_csv

print('\033[1mWelcome to use SHRCSVLoader - CSV file io System\033[0m\n|  \033[1;34mGithub : https://github.com/JNTMTMTM/SHICTHRS_CSVLoader\033[0m')
print('|  \033[1mAlgorithms = rule ; Questioning = approval\033[0m')
print('|  \033[1mCopyright : © 2025-2026 SHICTHRS, Std. All rights reserved.\033[0m\n')

__all__ = ['SHRCSVLoader_read_csv_file' , 'SHRCSVLoader_write_csv_file' , 'SHRCSVLoader_insert_csv_file']

class SHRCSVLoaderException(BaseException):
    def __init__(self , message: str) -> None:
        self.message = message
    
    def __str__(self):
        return self.message

def SHRCSVLoader_read_csv_file(path : str , read_encoding : str = 'GB2312') -> dict:
    try:
        if os.path.exists(path):
            if os.path.isfile(path) and (path.endswith('.csv') or path.endswith('.CSV')):
                return read_csv_file(path , read_encoding)
            else:
                raise SHRCSVLoaderException(f"SHRJsonLoader [ERROR.1017] only csv file is supported not .{path.split('.')[-1]}.")
        else:
            raise SHRCSVLoaderException(f"SHRJsonLoader [ERROR.1018] unable to find csv file. File Path : {path} NOT FOUND")
    except Exception as e:
        raise SHRCSVLoaderException(f"SHRJsonLoader [ERROR.1019] unable to read csv file. File Path : {path} | {e}")

def SHRCSVLoader_write_csv_file(data: dict, path: str, write_encoding: str = 'GB2312') -> bool:
    try:
        if not isinstance(data , dict):
            raise SHRCSVLoaderException("SHRCSVLoader [ERROR.1020] data must be a dictionary")
            
        if not (path.endswith('.csv') or path.endswith('.CSV')):
            raise SHRCSVLoaderException(f"SHRCSVLoader [ERROR.1021] only .csv file is supported not .{path.split('.')[-1]}.")
            
        result = write_csv_file(data, path, write_encoding)
        
        if not result:
            raise SHRCSVLoaderException(f"SHRCSVLoader [ERROR.1022] unable to write csv file. File Path : {path}")
            
    except Exception as e:
        raise SHRCSVLoaderException(f"SHRCSVLoader [ERROR.1023] unable to write csv file. File Path : {path} | {e}")

def SHRCSVLoader_insert_csv_file(path: str , header : dict , insert_encoding: str = 'GB2312') -> bool:
    try:
        if not isinstance(header , list):
            raise SHRCSVLoaderException("SHRCSVLoader [ERROR.1024] data must be a list")
            
        if not (path.endswith('.csv') or path.endswith('.CSV')):
            raise SHRCSVLoaderException(f"SHRCSVLoader [ERROR.1025] only .csv file is supported not .{path.split('.')[-1]}.")
                
        # 写入合并后的数据
        result = insert_header_to_csv(path , header , insert_encoding)
        
        if not result:
            raise SHRCSVLoaderException(f"SHRCSVLoader [ERROR.1026] unable to insert data to csv file. File Path : {path}")
            
    except Exception as e:
        raise SHRCSVLoaderException(f"SHRCSVLoader [ERROR.1027] unable to insert data to csv file. File Path : {path} | {e}")




import csv
import os

def write_csv_file(data: dict, path: str, write_encoding: str = 'GB2312') -> bool:
    """
    data: 要写入的数据，格式为{row_num: {column_name: value}}
    path: CSV文件路径
    write_encoding: 写入编码 默认为GB2312
    """

    # 确保目录存在
    try:
        directory = os.path.dirname(path)
        if directory and not os.path.exists(directory):
            os.makedirs(directory)
            
        # 如果数据为空，创建一个空文件
        if not data:
            with open(path, 'w', encoding=write_encoding, newline='') as f:
                f.write('')
            return True
            
        # 获取所有列名
        all_columns = set()
        for row in data.values():
            all_columns.update(row.keys())
        sorted_columns = sorted(all_columns)
        
        with open(path, 'w', encoding=write_encoding, newline='') as f:
            writer = csv.DictWriter(f, fieldnames=sorted_columns)
            writer.writeheader()
            
            # 按行号排序数据并写入
            for row_num in sorted(data.keys()):
                writer.writerow(data[row_num])
        return True
    except:
        return False



import os
import csv
from tempfile import NamedTemporaryFile

def insert_header_to_csv(file_path : str , header_row : list , insert_encoding):
    with NamedTemporaryFile(mode = 'w', delete = False, newline = '' , encoding = insert_encoding) as temp_file:
        temp_path = temp_file.name
        writer = csv.writer(temp_file)
        writer.writerow(header_row)
        with open(file_path , 'r' , newline = '', encoding = insert_encoding) as orig_file:
            reader = csv.reader(orig_file)
            for row in reader:
                writer.writerow(row)
        os.replace(temp_path , file_path)